@@ -13,7 +13,7 @@
 ** is not included in the SQLite library.  It is used for automated
 ** testing of the SQLite library.
 **
-** $Id: test3.c,v 1.14 2002/02/19 13:39:23 drh Exp $
+** $Id: test3.c,v 1.23 2003/04/13 18:26:52 paul Exp $
 */
 #include "sqliteInt.h"
 #include "pager.h"
@@ -57,7 +57,7 @@ static int btree_open(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int rc;
@@ -67,12 +67,15 @@ static int btree_open(
        " FILENAME\"", 0);
     return TCL_ERROR;
   }
-  rc = sqliteBtreeOpen(argv[1], 0666, 10, &pBt);
+  rc = sqliteBtreeFactory(0, argv[1], 0, 1000, &pBt);
   if( rc!=SQLITE_OK ){
     Tcl_AppendResult(interp, errorName(rc), 0);
     return TCL_ERROR;
   }
-  sprintf(zBuf,"0x%x",(int)pBt);
+  sprintf(zBuf,"%p", pBt);
+  if( strncmp(zBuf,"0x",2) ){
+    sprintf(zBuf, "0x%p", pBt);
+  }
   Tcl_AppendResult(interp, zBuf, 0);
   return TCL_OK;
 }
@@ -86,7 +89,7 @@ static int btree_close(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int rc;
@@ -113,7 +116,7 @@ static int btree_begin_transaction(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int rc;
@@ -140,7 +143,7 @@ static int btree_rollback(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int rc;
@@ -167,7 +170,7 @@ static int btree_commit(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int rc;
@@ -194,7 +197,7 @@ static int btree_create_table(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int rc, iTable;
@@ -224,7 +227,7 @@ static int btree_drop_table(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int iTable;
@@ -253,7 +256,7 @@ static int btree_clear_table(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int iTable;
@@ -282,7 +285,7 @@ static int btree_get_meta(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int rc;
@@ -316,7 +319,7 @@ static int btree_update_meta(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int rc;
@@ -351,7 +354,7 @@ static int btree_page_dump(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int iPage;
@@ -381,7 +384,7 @@ static int btree_tree_dump(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int iPage;
@@ -411,7 +414,7 @@ static int btree_pager_stats(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int i;
@@ -446,7 +449,7 @@ static int btree_pager_ref_dump(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
 
@@ -471,7 +474,7 @@ static int btree_integrity_check(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   char *zResult;
@@ -493,7 +496,7 @@ static int btree_integrity_check(
   zResult = sqliteBtreeIntegrityCheck(pBt, aRoot, nRoot);
   if( zResult ){
     Tcl_AppendResult(interp, zResult, 0);
-    free(zResult); 
+    sqliteFree(zResult); 
   }
   return TCL_OK;
 }
@@ -507,7 +510,7 @@ static int btree_cursor(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Btree *pBt;
   int iTable;
@@ -543,7 +546,7 @@ static int btree_close_cursor(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   BtCursor *pCur;
   int rc;
@@ -571,7 +574,7 @@ static int btree_move_to(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   BtCursor *pCur;
   int rc;
@@ -605,7 +608,7 @@ static int btree_delete(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   BtCursor *pCur;
   int rc;
@@ -634,7 +637,7 @@ static int btree_insert(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   BtCursor *pCur;
   int rc;
@@ -657,16 +660,20 @@ static int btree_insert(
 /*
 ** Usage:   btree_next ID
 **
-** Move the cursor to the next entry in the table.
+** Move the cursor to the next entry in the table.  Return 0 on success
+** or 1 if the cursor was already on the last entry in the table or if
+** the table is empty.
 */
 static int btree_next(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   BtCursor *pCur;
   int rc;
+  int res = 0;
+  char zBuf[100];
 
   if( argc!=2 ){
     Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
@@ -674,11 +681,113 @@ static int btree_next(
     return TCL_ERROR;
   }
   if( Tcl_GetInt(interp, argv[1], (int*)&pCur) ) return TCL_ERROR;
-  rc = sqliteBtreeNext(pCur, 0);
+  rc = sqliteBtreeNext(pCur, &res);
   if( rc ){
     Tcl_AppendResult(interp, errorName(rc), 0);
     return TCL_ERROR;
   }
+  sprintf(zBuf,"%d",res);
+  Tcl_AppendResult(interp, zBuf, 0);
+  return SQLITE_OK;
+}
+
+/*
+** Usage:   btree_prev ID
+**
+** Move the cursor to the previous entry in the table.  Return 0 on
+** success and 1 if the cursor was already on the first entry in
+** the table or if the table was empty.
+*/
+static int btree_prev(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  const char **argv      /* Text of each argument */
+){
+  BtCursor *pCur;
+  int rc;
+  int res = 0;
+  char zBuf[100];
+
+  if( argc!=2 ){
+    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
+       " ID\"", 0);
+    return TCL_ERROR;
+  }
+  if( Tcl_GetInt(interp, argv[1], (int*)&pCur) ) return TCL_ERROR;
+  rc = sqliteBtreePrevious(pCur, &res);
+  if( rc ){
+    Tcl_AppendResult(interp, errorName(rc), 0);
+    return TCL_ERROR;
+  }
+  sprintf(zBuf,"%d",res);
+  Tcl_AppendResult(interp, zBuf, 0);
+  return SQLITE_OK;
+}
+
+/*
+** Usage:   btree_first ID
+**
+** Move the cursor to the first entry in the table.  Return 0 if the
+** cursor was left point to something and 1 if the table is empty.
+*/
+static int btree_first(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  const char **argv      /* Text of each argument */
+){
+  BtCursor *pCur;
+  int rc;
+  int res = 0;
+  char zBuf[100];
+
+  if( argc!=2 ){
+    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
+       " ID\"", 0);
+    return TCL_ERROR;
+  }
+  if( Tcl_GetInt(interp, argv[1], (int*)&pCur) ) return TCL_ERROR;
+  rc = sqliteBtreeFirst(pCur, &res);
+  if( rc ){
+    Tcl_AppendResult(interp, errorName(rc), 0);
+    return TCL_ERROR;
+  }
+  sprintf(zBuf,"%d",res);
+  Tcl_AppendResult(interp, zBuf, 0);
+  return SQLITE_OK;
+}
+
+/*
+** Usage:   btree_last ID
+**
+** Move the cursor to the last entry in the table.  Return 0 if the
+** cursor was left point to something and 1 if the table is empty.
+*/
+static int btree_last(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  const char **argv      /* Text of each argument */
+){
+  BtCursor *pCur;
+  int rc;
+  int res = 0;
+  char zBuf[100];
+
+  if( argc!=2 ){
+    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
+       " ID\"", 0);
+    return TCL_ERROR;
+  }
+  if( Tcl_GetInt(interp, argv[1], (int*)&pCur) ) return TCL_ERROR;
+  rc = sqliteBtreeLast(pCur, &res);
+  if( rc ){
+    Tcl_AppendResult(interp, errorName(rc), 0);
+    return TCL_ERROR;
+  }
+  sprintf(zBuf,"%d",res);
+  Tcl_AppendResult(interp, zBuf, 0);
   return SQLITE_OK;
 }
 
@@ -691,7 +800,7 @@ static int btree_key(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   BtCursor *pCur;
   int rc;
@@ -729,7 +838,7 @@ static int btree_data(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   BtCursor *pCur;
   int rc;
@@ -758,6 +867,34 @@ static int btree_data(
   return SQLITE_OK;
 }
 
+/*
+** Usage:   btree_payload_size ID
+**
+** Return the number of bytes of payload
+*/
+static int btree_payload_size(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  const char **argv      /* Text of each argument */
+){
+  BtCursor *pCur;
+  int n1, n2;
+  char zBuf[50];
+
+  if( argc!=2 ){
+    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
+       " ID\"", 0);
+    return TCL_ERROR;
+  }
+  if( Tcl_GetInt(interp, argv[1], (int*)&pCur) ) return TCL_ERROR;
+  sqliteBtreeKeySize(pCur, &n1);
+  sqliteBtreeDataSize(pCur, &n2);
+  sprintf(zBuf, "%d", n1+n2);
+  Tcl_AppendResult(interp, zBuf, 0);
+  return SQLITE_OK;
+}
+
 /*
 ** Usage:   btree_cursor_dump ID
 **
@@ -777,7 +914,7 @@ static int btree_cursor_dump(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   BtCursor *pCur;
   int rc;
@@ -809,32 +946,47 @@ static int btree_cursor_dump(
 ** Register commands with the TCL interpreter.
 */
 int Sqlitetest3_Init(Tcl_Interp *interp){
-  Tcl_CreateCommand(interp, "btree_open", btree_open, 0, 0);
-  Tcl_CreateCommand(interp, "btree_close", btree_close, 0, 0);
-  Tcl_CreateCommand(interp, "btree_begin_transaction",
-      btree_begin_transaction, 0, 0);
-  Tcl_CreateCommand(interp, "btree_commit", btree_commit, 0, 0);
-  Tcl_CreateCommand(interp, "btree_rollback", btree_rollback, 0, 0);
-  Tcl_CreateCommand(interp, "btree_create_table", btree_create_table, 0, 0);
-  Tcl_CreateCommand(interp, "btree_drop_table", btree_drop_table, 0, 0);
-  Tcl_CreateCommand(interp, "btree_clear_table", btree_clear_table, 0, 0);
-  Tcl_CreateCommand(interp, "btree_get_meta", btree_get_meta, 0, 0);
-  Tcl_CreateCommand(interp, "btree_update_meta", btree_update_meta, 0, 0);
-  Tcl_CreateCommand(interp, "btree_page_dump", btree_page_dump, 0, 0);
-  Tcl_CreateCommand(interp, "btree_tree_dump", btree_tree_dump, 0, 0);
-  Tcl_CreateCommand(interp, "btree_pager_stats", btree_pager_stats, 0, 0);
-  Tcl_CreateCommand(interp, "btree_pager_ref_dump", btree_pager_ref_dump, 0, 0);
-  Tcl_CreateCommand(interp, "btree_cursor", btree_cursor, 0, 0);
-  Tcl_CreateCommand(interp, "btree_close_cursor", btree_close_cursor, 0, 0);
-  Tcl_CreateCommand(interp, "btree_move_to", btree_move_to, 0, 0);
-  Tcl_CreateCommand(interp, "btree_delete", btree_delete, 0, 0);
-  Tcl_CreateCommand(interp, "btree_insert", btree_insert, 0, 0);
-  Tcl_CreateCommand(interp, "btree_next", btree_next, 0, 0);
-  Tcl_CreateCommand(interp, "btree_key", btree_key, 0, 0);
-  Tcl_CreateCommand(interp, "btree_data", btree_data, 0, 0);
-  Tcl_CreateCommand(interp, "btree_cursor_dump", btree_cursor_dump, 0, 0);
-  Tcl_CreateCommand(interp, "btree_integrity_check", btree_integrity_check,0,0);
+  static struct {
+     char *zName;
+     Tcl_CmdProc *xProc;
+  } aCmd[] = {
+     { "btree_open",               (Tcl_CmdProc*)btree_open               },
+     { "btree_close",              (Tcl_CmdProc*)btree_close              },
+     { "btree_begin_transaction",  (Tcl_CmdProc*)btree_begin_transaction  },
+     { "btree_commit",             (Tcl_CmdProc*)btree_commit             },
+     { "btree_rollback",           (Tcl_CmdProc*)btree_rollback           },
+     { "btree_create_table",       (Tcl_CmdProc*)btree_create_table       },
+     { "btree_drop_table",         (Tcl_CmdProc*)btree_drop_table         },
+     { "btree_clear_table",        (Tcl_CmdProc*)btree_clear_table        },
+     { "btree_get_meta",           (Tcl_CmdProc*)btree_get_meta           },
+     { "btree_update_meta",        (Tcl_CmdProc*)btree_update_meta        },
+     { "btree_page_dump",          (Tcl_CmdProc*)btree_page_dump          },
+     { "btree_tree_dump",          (Tcl_CmdProc*)btree_tree_dump          },
+     { "btree_pager_stats",        (Tcl_CmdProc*)btree_pager_stats        },
+     { "btree_pager_ref_dump",     (Tcl_CmdProc*)btree_pager_ref_dump     },
+     { "btree_cursor",             (Tcl_CmdProc*)btree_cursor             },
+     { "btree_close_cursor",       (Tcl_CmdProc*)btree_close_cursor       },
+     { "btree_move_to",            (Tcl_CmdProc*)btree_move_to            },
+     { "btree_delete",             (Tcl_CmdProc*)btree_delete             },
+     { "btree_insert",             (Tcl_CmdProc*)btree_insert             },
+     { "btree_next",               (Tcl_CmdProc*)btree_next               },
+     { "btree_prev",               (Tcl_CmdProc*)btree_prev               },
+     { "btree_key",                (Tcl_CmdProc*)btree_key                },
+     { "btree_data",               (Tcl_CmdProc*)btree_data               },
+     { "btree_payload_size",       (Tcl_CmdProc*)btree_payload_size       },
+     { "btree_first",              (Tcl_CmdProc*)btree_first              },
+     { "btree_last",               (Tcl_CmdProc*)btree_last               },
+     { "btree_cursor_dump",        (Tcl_CmdProc*)btree_cursor_dump        },
+     { "btree_integrity_check",    (Tcl_CmdProc*)btree_integrity_check    },
+  };
+  int i;
+
+  for(i=0; i<sizeof(aCmd)/sizeof(aCmd[0]); i++){
+    Tcl_CreateCommand(interp, aCmd[i].zName, aCmd[i].xProc, 0, 0);
+  }
   Tcl_LinkVar(interp, "pager_refinfo_enable", (char*)&pager_refinfo_enable,
      TCL_LINK_INT);
+  Tcl_LinkVar(interp, "btree_native_byte_order",(char*)&btree_native_byte_order,
+     TCL_LINK_INT);
   return TCL_OK;
 }
