@@ -1,12 +1,12 @@
 #
 # Run this TCL script to generate HTML for the index.html file.
 #
-set rcsid {$Id: index.tcl,v 1.59 2002/04/12 13:11:53 drh Exp $}
+set rcsid {$Id: index.tcl,v 1.82 2004/04/23 17:04:45 drh Exp $}
 
 puts {<html>
-<head><title>SQLite: An SQL Database Engine In A C Library</title></head>
+<head><title>SQLite: An Embeddable SQL Database Engine</title></head>
 <body bgcolor=white>
-<h1 align=center>SQLite: An SQL Database Engine In A C Library</h1>
+<h1 align=center>SQLite<br>An Embeddable SQL Database Engine</h1>
 <p align=center>}
 puts "This page was last modified on [lrange $rcsid 3 4] UTC<br>"
 set vers [lindex $argv 0]
@@ -36,8 +36,9 @@ puts {
 Quick Links:
 <ul>
 <li><a href="download.html">Download</a></li>
-<li><a href="http://cvs.hwaci.com:2080/sqlite/timeline">Change Log</a></li>
-<li><a href="http://cvs.hwaci.com:2080/sqlite/tktnew">Report a bug</a></li>
+<li><a href="http://www.sqlite.org/cvstrac/timeline">Change Log</a></li>
+<li><a href="http://www.sqlite.org/cvstrac/tktnew">Report a bug</a></li>
+<li><a href="quickstart.html">Quick start</a></li>
 </ul>
 </td></tr></table>
 </td></tr>
@@ -47,21 +48,31 @@ Quick Links:
 puts {<h2>Features</h2>
 
 <p><ul>
-<li>Implements a large subset of SQL92.</li>
+<li>Implements most of SQL92.
+    (<a href="omitted.html">Features not supported</a>)</li>
 <li>A complete database (with multiple tables and indices) is
     stored in a single disk file.</li>
-<li>Atomic commit and rollback protect data integrity.</li>
-<li>Small memory footprint: less than 20K lines of C code.</li>
-<li><a href="speed.html">Four times faster</a> than PostgreSQL.
-    Twice as fast as SQLite 1.0.</li>
+<li>ACID (Atomic, Consistent, Isolated, Durable) transactions.</li>
+<li>Database files can be freely shared between machines with
+    different byte orders.</li>
+<li>Supports databases up to 2 terabytes (2^41 bytes) in size.</li>
+<li>A zero-configuration database engine - no DBA needed.</li>
+<li>Small memory footprint: less than 30K lines of C code.</li>
+<li><a href="speed.html">Two times faster</a> than PostgreSQL and
+    MySQL for many common operations.</li>
 <li>Very simple 
 <a href="c_interface.html">C/C++ interface</a> requires the use of only
 three functions and one opaque structure.</li>
-<li><a href="tclsqlite.html">TCL bindings</a> included.</li>
-<li>A TCL-based test suite provides near 100% code coverage.</li>
+<li><a href="tclsqlite.html">TCL bindings</a> included.
+    Bindings for many other languages 
+    <a href="http://www.sqlite.org/cvstrac/wiki?p=SqliteWrappers">
+    available separately.</a></li>
+<li>Simple, well-commented source code.</li>
+<li>Automated test suite provides over 90% code coverage.</li>
 <li>Self-contained: no external dependencies.</li>
-<li>Built and tested under Linux and Win2K.</li>
-<li>Sources are uncopyrighted.  Use for any purpose.</li>
+<li>Built and tested under Linux and Windows.</li>
+<li>Sources are in the <a href="copyright.html">public domain</a>.
+    Use for any purpose.</li>
 </ul>
 </p>
 }
@@ -70,75 +81,85 @@ puts {<h2>Current Status</h2>
 
 <p>A <a href="changes.html">Change Summary</a> is available on this
 website.  You can also access a detailed
-<a href="http://cvs.hwaci.com:2080/sqlite/timeline">change history</a>,
-<a href="http://cvs.hwaci.com:2080/sqlite/rptview?rn=2">view open bugs</a>,
+<a href="http://www.sqlite.org/cvstrac/timeline">change history</a>,
+<a href="http://www.sqlite.org/cvstrac/rptview?rn=2">view open bugs</a>,
 or
-<a href="http://cvs.hwaci.com:2080/sqlite/tktnew">report new bugs</a>
+<a href="http://www.sqlite.org/cvstrac/tktnew">report new bugs</a>
 at the
-<a href="http://cvs.hwaci.com:2080/sqlite/">CVS server</a>.</p>
+<a href="http://www.sqlite.org/cvstrac/">CVS server</a>.</p>
 
 <p>Complete source code and precompiled binaries for the latest release are
 <a href="download.html">available for download</a> on this site.  You
 can also obtain the latest changes by anonymous CVS access:
 <blockquote><pre>
-cvs -d :pserver:anonymous@cvs.hwaci.com:/home/cvs/sqlite login
-cvs -d :pserver:anonymous@cvs.hwaci.com:/home/cvs/sqlite checkout sqlite
+cvs -d :pserver:anonymous@www.sqlite.org:/sqlite login
+cvs -d :pserver:anonymous@www.sqlite.org:/sqlite checkout sqlite
 </pre></blockquote>
 When prompted for a password, enter "anonymous".
 </p>
 
-<p>Note that the CVS server is located on a cable modem with a dynamic
-IP address.  The IP address changes every 3 or 4 months.  After an
-IP address change
-occurs it usually takes a day or two for the new DNS information to propagate.
-So if you have trouble accessing the CVS server, it could be because the
-IP address has recently changed.  Try again in a few days.
-</p>
-
 <p>
 Whenever either of the first two digits in the version number
 for SQLite change, it means that the underlying file format
-has changed.  See <a href="formatchng.html">formatchng.html</a>
+has changed.  Usually these changes are backwards compatible.
+See <a href="formatchng.html">formatchng.html</a>
 for additional information.
 </p>
+}
 
-<h2>Documentation</h2>
+puts {<h2>Documentation</h2>
 
 <p>The following documentation is currently available:</p>
 
 <p><ul>
+<li>A <a href="quickstart.html">Quick Start</a> guide to using SQLite in
+    5 minutes or less.</li>
 <li><a href="faq.html">Frequently Asked Questions</a> are available online.</li>
 <li>Information on the <a href="sqlite.html">sqlite</a>
     command-line utility.</li>
+<li>SQLite is <a href="datatypes.html">typeless</a>.
 <li>The <a href="lang.html">SQL Language</a> subset understood by SQLite.</li>
 <li>The <a href="c_interface.html">C/C++ Interface</a>.</li>
+<li>The <a href="nulls.html">NULL handling</a> in SQLite versus
+    other SQL database engines.</li>
 <li>The <a href="tclsqlite.html">Tcl Binding</a> to SQLite.</li>
 <li>The <a href="arch.html">Architecture of the SQLite Library</a> describes
     how the library is put together.</li>
 <li>A description of the <a href="opcode.html">virtual machine</a> that
     SQLite uses to access the database.</li>
+<li>A description of the 
+    <a href="fileformat.html">database file format</a> used by SQLite.
 <li>A <a href="speed.html">speed comparison</a> between SQLite, PostgreSQL,
     and MySQL.</li>
+<li>User-written documentation is available on the
+    <a href="http://www.sqlite.org/cvstrac/wiki">SQLite Wiki</a>.  Please
+    contribute if you can.</li>
 </ul>
 </p>
 
-<p>The SQLite source code is 35% comment.  These comments are
-another important source of information. </p>
+<p>The SQLite source code is 30% comment.  These comments are
+another important source of information.  </p>
 
 }
 
 puts {
-<table align="right">
-<tr><td align="center">
-<a href="http://www.yahoogroups.com/subscribe/sqlite">
-<img src="http://www.egroups.com/img/ui/join.gif" border=0 /><br />
-Click to subscribe to sqlite</a>
-</td></tr>
-</table>
 <a name="mailinglist" />
 <h2>Mailing List</h2>
-<p>A mailing list has been set up on yahooGroups for discussion of
-SQLite design issues or for asking questions about SQLite.</p>
+<p>A mailing list has been set up for discussion of
+SQLite design issues or for asking questions about SQLite.
+To subscribe send an email to
+<a href="mailto:sqlite-users-subscribe@sqlite.org">
+sqlite-users-subscribe@sqlite.org</a>.
+If you would prefer to get digests rather than individual
+emails, send a message to to
+<a href="mailto:sqlite-users-digest-subscribe@sqlite.org">
+sqlite-users-digest-subscribe@sqlite.org</a>.
+For additional information about operating and using this
+mailing list, send a message to
+<a href="mailto:sqlite-users-help@sqlite.org">
+sqlite-users-help@sqlite.org</a> and instructions will be
+sent by to you by return email.
+</p>
 }
 
 puts {<h2>Professional Support and Custom Modifications</h2>}
@@ -146,9 +167,11 @@ puts {<h2>Professional Support and Custom Modifications</h2>}
 puts {
 <p>
 If you would like professional support for SQLite
-or if you want custom modifications to SQLite preformed by the
+or if you want custom modifications to SQLite performed by the
 original author, these services are available for a modest fee.
-For additional information contact:</p>
+For additional information visit
+<a href="http://www.hwaci.com/sw/sqlite/support.html">
+http://www.hwaci.com/sw/sqlite/support.html</a> or contact:</p>
 
 <blockquote>
 D. Richard Hipp <br />
@@ -173,26 +196,28 @@ $ ../sqlite/configure
 $ make                       <i> Builds "sqlite" and "libsqlite.a" </i>
 $ make test                  <i> Optional: run regression tests </i>
 </pre></blockquote>
+
+<p>If you prefer, you can also build by making whatever modifications
+you desire to the file "Makefile.linux-gcc" and then executing that
+makefile.  The latter method is used for all official development
+and testing of SQLite and for building the precompiled
+binaries found on this website.  Windows binaries are generated by
+cross-compiling from Linux using <a href="http://www.mingw.org/">MinGW</a></p>
 }
 
 puts {<h2>Related Sites</h2>
 
-<ul>
-
-<li><p>An ODBC driver for SQLite can be found at
-       <a href="http://www.ch-werner.de/sqliteodbc/">
-       http://www.ch-werner.de/sqliteodbc/</a>.</p></li>
-
-<li><p>A PHP module for SQLite can be found at
-       <a href="http://freshmeat.net/projects/sqlite-php">
-       http://freshmeat.net/projects/sqlite-php</a></li>
-</ul>}
-
-puts {
-<p><hr /></p>
 <p>
-<a href="../index.html"><img src="/goback.jpg" border=0 />
-More Open Source Software</a> from Hwaci.
+For information bindings of SQLite to other programming languages
+(Perl, Python, Ruby, PHP, etc.) and for a list of programs currently
+using SQLite, visit the Wiki documentation at:
 </p>
 
+<blockquote>
+<a href="http://www.sqlite.org/cvstrac/wiki">
+http://www.sqlite.org/cvstrac/wiki</a>
+</blockquote>
+}
+
+puts {
 </body></html>}
