@@ -1,7 +1,7 @@
 #
 # Run this Tcl script to generate the sqlite.html file.
 #
-set rcsid {$Id: sqlite.tcl,v 1.16 2001/11/24 13:23:05 drh Exp $}
+set rcsid {$Id: sqlite.tcl,v 1.21 2003/06/29 16:11:13 drh Exp $}
 
 puts {<html>
 <head>
@@ -35,14 +35,16 @@ with a single table named "tbl1", you might do this:</p>
 }
 
 proc Code {body} {
-  puts {<blockquote><pre>}
+  puts {<blockquote><tt>}
   regsub -all {&} [string trim $body] {\&amp;} body
   regsub -all {>} $body {\&gt;} body
   regsub -all {<} $body {\&lt;} body
-  regsub -all {\(\(\(} $body {<font color="#00671f"><u>} body
-  regsub -all {\)\)\)} $body {</u></font>} body
+  regsub -all {\(\(\(} $body {<b>} body
+  regsub -all {\)\)\)} $body {</b>} body
+  regsub -all { } $body {\&nbsp;} body
+  regsub -all \n $body <br>\n body
   puts $body
-  puts {</pre></blockquote>}
+  puts {</tt></blockquote>}
 }
 
 Code {
@@ -59,10 +61,6 @@ sqlite>
 }
 
 puts {
-<p>(In the example above, and in all subsequent examples, the commands
-you type are underlined and shown with a green tint and the responses
-from the computer are shown in black without underlining.)</p>
-
 <p>You can terminate the sqlite program by typing your systems
 End-Of-File character (usually a Control-D) or the interrupt
 character (usually a Control-C).</p>
@@ -117,6 +115,14 @@ indices from the database.  You can not make manual changes
 to the sqlite_master table.
 </p>
 
+<p>
+The schema for TEMPORARY tables is not stored in the "sqlite_master" table
+since TEMPORARY tables are not visible to applications other than the
+application that created the table.  The schema for TEMPORARY tables
+is stored in another special table named "sqlite_temp_master".  The
+"sqlite_temp_master" table is temporary itself.
+</p>
+
 <h2>Special commands to sqlite</h2>
 
 <p>
@@ -135,19 +141,27 @@ at any time.  For example:
 
 Code {
 sqlite> (((.help)))
-.dump                  Dump database in a text format
+.databases             List names and files of attached databases
+.dump ?TABLE? ...      Dump the database in a text format
+.echo ON|OFF           Turn command echo on or off
 .exit                  Exit this program
-.explain               Set output mode suitable for EXPLAIN
-.header ON|OFF         Turn display of headers on or off
+.explain ON|OFF        Turn output mode suitable for EXPLAIN on or off.
+.header(s) ON|OFF      Turn display of headers on or off
 .help                  Show this message
 .indices TABLE         Show names of all indices on TABLE
-.mode MODE             Set mode to one of "line", "column", "list", or "html"
+.mode MODE             Set mode to one of "line(s)", "column(s)", 
+                       "insert", "list", or "html"
 .mode insert TABLE     Generate SQL insert statements for TABLE
+.nullvalue STRING      Print STRING instead of nothing for NULL data
 .output FILENAME       Send output to FILENAME
 .output stdout         Send output to the screen
+.prompt MAIN CONTINUE  Replace the standard prompts
+.quit                  Exit this program
+.read FILENAME         Execute SQL in FILENAME
 .schema ?TABLE?        Show the CREATE statements
 .separator STRING      Change separator string for "list" mode
-.tables                List names all tables in the database
+.show                  Show the current values for various settings
+.tables ?PATTERN?      List names of tables matching a pattern
 .timeout MS            Try opening locked tables for MS milliseconds
 .width NUM NUM ...     Set column widths for "column" mode
 sqlite> 
@@ -158,7 +172,7 @@ puts {
 
 <p>The sqlite program is able to show the results of a query
 in five different formats: "line", "column", "list", "html", and "insert".
-You can use the ".mode" dot command to switch between these three output
+You can use the ".mode" dot command to switch between these output
 formats.</p>
 
 <p>The default output mode is "list".  In
@@ -335,8 +349,8 @@ puts {
 executing the following query:</p>
 
 <blockquote><pre>
-SELECT name FROM sqlite_master 
-WHERE type='table' 
+SELECT name FROM sqlite_master WHERE type='table' 
+UNION ALL SELECT name FROM sqlite_temp_master WHERE type='table'
 ORDER BY name;
 </pre></blockquote>
 
@@ -376,7 +390,9 @@ puts {
 list mode, then entering the following query:</p>
 
 <blockquote><pre>
-SELECT sql FROM sqlite_master
+SELECT sql FROM 
+   (SELECT * FROM sqlite_master UNION ALL
+    SELECT * FROM sqlite_temp_master)
 WHERE type!='meta'
 ORDER BY tbl_name, type DESC, name
 </pre></blockquote>
@@ -385,7 +401,9 @@ ORDER BY tbl_name, type DESC, name
 want the schema for a single table, the query looks like this:</p>
 
 <blockquote><pre>
-SELECT sql FROM sqlite_master
+SELECT sql FROM
+   (SELECT * FROM sqlite_master UNION ALL
+    SELECT * FROM sqlite_temp_master)
 WHERE tbl_name LIKE '%s' AND type!='meta'
 ORDER BY type DESC, name
 </pre></blockquote>
@@ -410,6 +428,19 @@ appended and prepended and a LIKE clause is added to the query.
 This allows you to list only those tables that match a particular
 pattern.</p>
 
+<p>The ".databases" command shows a list of all databases open in
+the current connection.  There will always be at least 2.  The first
+one is "main", the original database opened.  The second is "temp",
+the database used for temporary tables. There may be additional 
+databases listed for databases attached using the ATTACH statement.
+The first output column is the name the database is attached with, 
+and the second column is the filename of the external file.</p>}
+
+Code {
+sqlite> (((.databases)))
+}
+
+puts {
 <h2>Converting An Entire Database To An ASCII Text File</h2>
 
 <p>Use the ".dump" command to convert the entire contents of a
@@ -532,6 +563,18 @@ $ (((sqlite ex1 'select * from tbl1' |)))
 $
 }
 
+puts {
+<h2>Ending shell commands</h2>
+
+<p>
+SQLite commands are normally terminated by a semicolon.  In the shell 
+you can also use the word "GO" (case-insensitive) or a backslash character 
+"\" on a line by itself to end a command.  These are used by SQL Server 
+and Oracle, respectively.  These won't work in <b>sqlite_exec()</b>, 
+because the shell translates these into a semicolon before passing them 
+to that function.</p>
+}
+
 puts {
 <h2>Compiling the sqlite program from sources</h2>
 
